#ifndef __HASTORE_H__
#define __HASTORE_H__

#include "manager.h"
#include <new>

#ifndef NO_NAMESPACES
using namespace std;
#endif

#define REDEFINE_GLOBAL_NEW

#ifdef REDEFINE_GLOBAL_NEW
inline void* operator new(size_t size)  throw (std::bad_alloc) { 
    haManager* mng = haManager::getCurrent();
    if (mng != NULL) {
	return mng->allocate(haObjectHeader::CID_UDT, size);
    } else { 
	return malloc(size);
    }
}
inline void* operator new(size_t size, size_t varying) throw (std::bad_alloc) { 
    haManager* mng = haManager::getCurrent();
    if (mng != NULL) {
	return mng->allocate(haObjectHeader::CID_UDT, size + varying);
    } else { 
	return malloc(size + varying);
    }

}
inline void operator delete(void* ptr, size_t) 
throw() 
{
    haManager* mng = haManager::getManager((char*)ptr);
    if (mng != NULL) { 
	mng->deallocate(ptr);
    } else { 
	free(ptr);
    }
}
#endif

inline void* operator new(size_t size, haManager& mng, size_t varying = 0) { 
    return mng.allocate(haObjectHeader::CID_UDT, size + varying);
}

inline void operator delete(void* ptr) 
throw() 
{
    haManager* mng = haManager::getManager((char*)ptr);
    if (mng != NULL) { 
	mng->deallocate(ptr);
    } else { 
	free(ptr);
    }
}

inline void operator delete(void* ptr, haManager& m, size_t varying) 
throw() 
{
    haManager* mng = haManager::getManager((char*)ptr);
    if (mng != NULL) { 
	mng->deallocate(ptr);
    } else { 
	free(ptr);
    }
}

//
// HAS uses page protection mechanism to detect dirty pages.
// This exception is catched by unhandled exception filter. But if you want to use debugger, 
// you shoud use structured exception handling. You should always use
// structured exception handling with Borland C++, because Unhandled Exception 
// Filter is not correctly called in Borland. 
// Please use two following macros to enclose body of main (or WinMain)
// function:
//
#if defined(_WIN32)
#define SEN_TRY __try
#define SEN_ACCESS_VIOLATION_HANDLER() \
__except(GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION \
	&& haManager::pageFaultHandler((char*)(GetExceptionInformation())-> \
			      ExceptionRecord->ExceptionInformation[1]) \
  ? EXCEPTION_CONTINUE_EXECUTION \
  : EXCEPTION_CONTINUE_SEARCH) {}

#else // Unix
#define SEN_TRY
#define SEN_ACCESS_VIOLATION_HANDLER()
#endif

#endif
