#include "sockio.h"
#include "haexcept.h"

#ifdef _WIN32
#include <windows.h>

class WinSocketLibrary { 
  public:
    WinSocketLibrary() { 
	WSADATA wsa;
	if (WSAStartup(MAKEWORD(1, 1), &wsa) != 0) {
	    throw haException("Failed to initialize windows sockets",
			      WSAGetLastError());
	}
    }
    ~WinSocketLibrary() {
	//	WSACleanup();
    }
};

static WinSocketLibrary ws32Lib;

#define SOCK_ERROR WSAGetLastError()

#else

#if defined(__FreeBSD__) || defined(__linux__)
#include <sys/ioctl.h>
#else
#include <stropts.h>
#endif
#include <fcntl.h>
#include <sys/time.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/utsname.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stddef.h>
#include <assert.h>
#include <errno.h>
#include <signal.h>


extern "C" {
#include <netdb.h>
}

#define SOCK_ERROR errno
#define INVALID_SOCKET -1
#define closesocket(s) ::close(s)

#endif

haSocket::haSocket(int port)
{
    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {     
	throw haException("Failed to create socket", SOCK_ERROR);
    }
    struct sockaddr_in insock;
    insock.sin_family = AF_INET;
    insock.sin_addr.s_addr = htonl(INADDR_ANY);
    insock.sin_port = htons(port);
 
    int on = 1;
    setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char*)&on, sizeof on);

    if (bind(sock, (sockaddr*)&insock, sizeof(insock)) != 0) { 
	throw haException("Failed to bind socket", SOCK_ERROR);
    }
    if (listen(sock, 1) != 0) {
	throw haException("Failed to listen socket", SOCK_ERROR);
    }
 }

haSocket::~haSocket()
{
    closesocket(sock);
}

haSocket* haSocket::connect(char const* host, int port)
{
    struct sockaddr_in insock;  // inet socket address
    struct hostent*    hp;      // entry in hosts table
    int    s;

    if ((hp = gethostbyname(host)) == NULL || hp->h_addrtype != AF_INET) {
	throw haException("Failed to get host by name", SOCK_ERROR);
    }
    insock.sin_family = AF_INET;
    insock.sin_port = htons(port);
    
    for (int i = 0; hp->h_addr_list[i] != NULL; i++) { 
	memcpy(&insock.sin_addr, hp->h_addr_list[i],
	       sizeof insock.sin_addr);
	if ((s = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) { 
	    throw haException("Failed to create socket", SOCK_ERROR);
	}
	if (::connect(s, (sockaddr*)&insock, sizeof insock) == 0) { 
	    int enabled = 1;
	    setsockopt(s, IPPROTO_TCP, TCP_NODELAY, (char*)&enabled, sizeof enabled);
	    haSocket* ns = new haSocket();
	    ns->sock = s;
	    return ns;
	}
	closesocket(s);
    }
    return NULL;
}

haSocket* haSocket::accept() 
{
    int sd = ::accept(sock, NULL, NULL);
    if (sd < 0) { 
	return NULL;
    } else { 
	haSocket* s = new haSocket();
	s->sock = sd;
	return s;
    }
}

bool haSocket::read(void* buf, size_t size)
{
    char* dst = (char*)buf;
    while (size > 0) { 
	int rc = recv(sock, dst, size, 0);
	if (rc <= 0) {
#ifndef _WIN32
	    if (errno == EINTR) { 
		continue;
	    }
#endif	    
	    return false;
	}
	size -= rc;
	dst += rc;
    }
    return true;
}


bool haSocket::write(void const* buf, size_t size)
{
    char* src = (char*)buf;
    while (size > 0) { 
	int rc = send(sock, src, size, 0);
	if (rc <= 0) {
#ifndef _WIN32
	    if (errno == EINTR) { 
		continue;
	    }
#endif	    
	    return false;
	}
	size -= rc;
	src += rc;
    }
    return true;
}

  








